
# q-expansion of newform 2888.2.a.s, downloaded from the LMFDB on 02 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 2888
weight = 2
poly_data = [-1, -3, 0, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0, 0], 1], [[0, 1, 0], 1], [[-2, 0, 1], 1]]

hecke_ring_character_values = None
aps_data = [[0, 0, 0], [1, 2, -1], [-1, 0, -1], [-2, -1, 0], [-2, -1, 1], [0, -2, 3], [-2, 3, -3], [0, 0, 0], [2, -2, 4], [-5, 0, 1], [1, -3, 0], [-4, -2, 1], [0, -5, 0], [2, -1, -3], [0, -3, -1], [6, -3, -3], [3, -2, 6], [-5, -4, 4], [-2, 2, -10], [-2, -10, 8], [0, 8, -4], [-7, 5, -4], [-4, 0, 1], [-5, -1, 3], [-7, 2, 2], [-1, -4, 3], [2, 6, -1], [6, 7, -2], [13, 2, -4], [1, 3, -8], [-1, -1, -2], [5, 3, -4], [2, 1, -7], [0, 9, 0], [-8, 1, 4], [-6, 6, -7], [-19, 0, -3], [-10, -11, 7], [-5, 4, -2], [-2, 12, -6], [7, 1, -7], [6, -4, -2], [-3, -3, 7], [9, -2, 13], [10, 3, 6], [-4, 12, -4], [7, 1, -7], [9, 2, 1], [8, 3, 2], [7, 4, -6], [7, 3, 11], [11, 0, 5], [-11, 5, -6], [6, 0, 3], [3, 10, -13], [-5, 3, 5], [-20, 7, -5], [5, 3, 3], [1, -8, 11], [-8, 5, -6], [-15, 5, -10], [3, 1, 15], [15, -3, 3], [-7, -3, 5], [-14, 6, 3], [-8, -11, -1], [5, -4, 8], [-9, 4, -11], [-5, 11, -8], [0, -11, 18], [-15, 5, -18], [6, -8, -7], [-3, 6, -22], [-2, -11, 9], [-5, -5, 2], [-6, -1, -19], [0, -1, 13], [-20, -1, 10], [-7, 9, 12], [-7, 6, 10], [19, -7, -9], [-10, 0, -2], [7, -3, 7], [-13, -1, -4], [4, -7, 4], [-3, -2, 10], [-1, -15, 4], [-20, -7, 5], [-9, -3, -14], [-9, 22, 6], [-15, 9, 3], [-2, 19, -10], [15, 1, 11], [25, -1, -6], [7, -12, -12], [-1, -18, 17], [-7, -19, 12], [-2, 0, 4], [-18, -7, -5], [-23, -5, -3], [-34, 4, 2], [13, -18, 1], [3, 3, -16], [12, 4, -5], [24, -8, 2], [1, -18, 3], [2, 27, -14], [12, -6, -8], [-21, 13, -4], [21, -13, 6], [-16, -1, 4], [2, 6, -18], [-3, -11, -5], [6, 18, -16], [14, -8, 2], [-2, -1, 0], [-21, 6, -5], [-9, 12, 9], [-1, 16, 0], [13, 11, -1], [26, 6, 1], [4, 3, 1], [-8, 11, -27], [-4, -2, -3], [4, -16, 24], [15, 5, -3], [-5, 9, 0], [39, 5, -1], [-1, -9, 1], [-1, 18, -4], [-3, -16, 13], [7, -8, 6], [-11, -21, 3], [-26, 5, -20], [13, -17, 0], [-31, -3, -9], [-30, -4, -3], [26, -13, -7], [-3, 21, -3], [15, -7, 8], [16, 13, -16], [16, 13, -15], [-16, -9, 1], [27, -8, 4], [3, -11, 25], [-9, 16, 18], [5, -25, 7], [-4, 5, -12], [-43, -3, 3], [-6, -10, 17], [37, -10, -5], [-7, 7, -11], [-23, -3, 0], [-3, 11, -22], [-1, 23, -21], [12, 4, -15], [-29, 13, 0], [-25, -9, 4], [-37, -9, 10], [21, 0, -6], [-34, 3, 7], [-2, 16, 11], [4, -4, 20], [6, 24, 0], [-3, 2, -10], [-7, 1, 29], [-4, 14, 17], [-16, 10, 13], [18, -10, -18], [4, 12, -8], [10, -7, 11], [2, 10, -4], [33, 5, -12], [-15, 11, 0], [-12, 14, 8], [-15, -11, -16], [-10, 6, -11], [11, 13, -18], [-20, 4, -20], [-19, -13, 23], [5, -3, 25], [1, -10, -9], [6, 14, 1], [17, 8, 9], [10, 8, 11], [-37, -4, -12], [12, 9, 4], [35, -3, 4], [-24, 7, 9], [5, -11, -20], [6, -14, 20], [-11, 19, 20], [8, 23, 3], [-21, 9, 0], [11, -29, 16], [28, -10, -3], [10, -2, 30], [-9, 3, -6], [-44, -7, 0], [16, -16, 0], [39, -11, 7], [21, 7, -13], [3, 11, -5], [-7, -19, -10], [1, 42, -11], [5, 31, -18], [-11, -6, -21], [8, 4, 21], [-11, -28, 12], [-8, -4, 28], [22, -4, -22], [4, 36, -32], [1, 6, 4], [7, -4, -11], [-13, 22, -26], [-31, -7, 2], [-10, -17, -14], [-9, 10, -31], [21, -26, 26], [21, 7, 11], [-3, -25, -1], [-27, -20, 14], [-1, 16, 11], [-19, 8, -31], [-11, -38, 30], [-5, -34, 27], [-12, -14, 0], [-41, -19, 7], [48, 13, -10], [-22, 13, -16], [-13, 35, -2], [22, -13, 11], [-31, -3, -1], [-19, -1, 20], [-28, 9, -12], [-49, 8, -8], [11, -1, 5], [21, 21, 3], [6, 32, -25], [-14, -14, 23], [22, 8, -7], [14, -2, 11], [18, -12, 7], [29, 3, 2], [14, -9, 8], [2, -32, -3], [12, -24, 7], [23, 8, -5], [19, -15, 11], [-28, 18, -13], [-48, -23, 15], [4, 6, 4], [-47, 2, -11], [13, -12, 9], [7, 2, 20], [-1, 19, 15], [-3, 24, -39], [-21, 6, -14], [-10, -2, -15], [-43, -4, 8], [40, -7, 26], [21, 15, -18], [13, 1, 35], [-27, -50, 28], [-18, -13, 11], [27, 20, -8], [11, 14, 25], [-19, -22, 32], [-24, -16, 19], [27, 14, -31], [-4, -28, 20], [-44, -9, 3], [4, -13, 3], [27, 29, -5], [-1, -16, -12], [-30, -5, 16], [3, 11, 27], [-17, -21, -3], [-6, 45, -7], [-8, -22, 18], [-59, 8, 6], [25, -33, 1], [-10, 11, -18], [-11, -19, 14], [3, -18, 32], [-13, -5, -19], [9, -20, -2], [-45, 8, 7], [6, -10, 46], [-11, -6, 22], [-3, 15, -20], [31, 1, -21], [-43, 2, -23], [24, -7, 2], [14, 30, -3], [18, 19, -35], [-24, 39, 17], [35, 38, -32], [40, -15, 4], [-2, 35, 3], [8, -14, 32], [13, 5, -18], [-13, 22, -37], [-38, 8, -12], [-7, 2, -5], [13, -11, 12], [-4, 12, -22], [-15, 37, -30], [31, -24, 26], [1, 33, -11], [13, -39, 23], [-14, -32, 30], [16, 15, 5], [-11, -18, 31], [-12, -20, 16], [-39, 13, -2], [-69, -21, 14], [15, -18, 6], [-14, 21, -10], [-15, -6, -6], [29, 15, 7], [-2, -11, 10], [-19, 2, -33], [-26, -32, 25], [0, 2, 4], [22, -4, 15], [0, -8, 2], [-9, -2, 9], [-15, -27, 6], [1, 13, -32], [42, 19, -19], [51, -1, 16], [11, 34, -28], [7, -4, 12], [-47, 23, -12], [-1, -3, -20], [29, -31, 7], [-23, -28, 9], [2, 2, -9], [-28, -10, 3], [0, 3, -28], [-38, -20, 24], [47, -12, 37], [-9, 12, -43], [-57, -34, 27], [-33, -12, 9], [-42, 7, -8], [4, -34, 9], [5, 17, -22], [11, 29, -2], [7, -16, -4], [39, 0, 21], [-6, -52, 9], [-16, -33, 14], [-3, 33, -47], [4, -22, 1], [14, -43, -5], [12, 2, 35], [53, 22, -21], [-52, -31, 10], [-24, 5, -24], [41, 20, -3], [-2, 24, 4], [62, -17, 24], [30, -28, -20], [7, -41, 42], [-29, -36, 6], [-7, 20, 8], [-33, -10, 4], [3, -31, 38], [-1, -19, 14], [43, -12, -1], [39, 4, -14], [-4, -42, 4], [-4, 17, -42], [5, -40, 27], [31, -4, 9], [34, 5, -13], [-39, -5, 15], [-53, -6, -15], [5, -12, -13], [26, 19, 17], [-48, -11, -4], [-29, 30, -65], [31, 14, 15], [50, 3, -20], [-6, 7, -4], [25, -27, 17], [13, -13, 5], [12, 33, -19], [-45, 16, -3], [-51, -4, 12], [-7, 23, 16], [-3, -11, -30], [38, -32, 18], [-19, -21, 28], [21, -25, -3], [-6, 55, -32], [-59, -3, 20], [66, -3, 1], [16, 2, -26], [15, 25, -17], [28, -26, 7], [37, -10, 7], [-8, 2, -17], [-2, -6, -6], [-13, 25, -34], [41, -23, 11], [6, 5, -3], [14, -53, 6], [5, 13, -5], [14, -6, -10], [11, -41, -12], [15, -22, 25], [8, -18, -17], [20, 67, -51], [-49, 30, -13], [-17, -8, -6], [5, -5, 13], [24, -4, -32], [17, 36, -21], [-54, 23, 9], [23, -38, 29], [14, 0, -30], [-29, 5, -17], [-16, -14, 24], [37, 0, -6], [16, -45, 10], [-8, 5, 11], [28, -2, -42]]
